package cn.skcks.docking.gb28181.wvp.service.video;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.IdUtil;
import jakarta.servlet.AsyncContext;
import jakarta.servlet.ServletResponse;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.bytedeco.ffmpeg.global.avcodec;
import org.bytedeco.ffmpeg.global.avutil;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.FFmpegFrameRecorder;
import org.bytedeco.javacv.Frame;
import org.bytedeco.javacv.FrameGrabber;
import org.springframework.stereotype.Service;

import java.io.*;
import java.nio.file.Path;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

@Slf4j
@Service
public class RecordService {
    public void header(HttpServletResponse response) {
        response.setContentType("video/mp4");
        response.setHeader("Accept-Ranges", "none");
        response.setHeader("Connection", "close");
    }

    @SneakyThrows
    public void record(HttpServletRequest request, HttpServletResponse response, String url, long timeout) {
        AsyncContext asyncContext = request.startAsync();
        asyncContext.setTimeout(0);
        asyncContext.start(() -> {
            header(response);
            record(asyncContext.getResponse(), url, timeout);
            log.info("record 结束");
            asyncContext.complete();
        });
    }

    @SneakyThrows
    public void record(ServletResponse response, String url, long timeout) {
        //        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
        Path tmp = Path.of(System.getProperty("java.io.tmpdir"), IdUtil.getSnowflakeNextIdStr()).toAbsolutePath();
        File file = new File(tmp + ".mp4");
        log.info("创建文件 {}, {}", file, file.createNewFile());

        log.info("url {}", url);
        FFmpegFrameGrabber grabber = new FFmpegFrameGrabber(url);
        grabber.start();
        FFmpegFrameRecorder recorder = new FFmpegFrameRecorder(file, grabber.getImageWidth(), grabber.getImageHeight(), grabber.getAudioChannels());
        recorder.start();
        log.info("开始录像");
        log.info("{}", file);
        recorder.setVideoCodec(avcodec.AV_CODEC_ID_H264);
        recorder.setPixelFormat(avutil.AV_PIX_FMT_YUV420P); // 视频源数据yuv
        recorder.setAudioCodec(avcodec.AV_CODEC_ID_AAC); // 设置音频压缩方式
        recorder.setFormat("mp4");
        recorder.setVideoOption("threads", String.valueOf(Runtime.getRuntime().availableProcessors())); // 解码线程数

        ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
        AtomicBoolean record = new AtomicBoolean(true);
        scheduledExecutorService.schedule(() -> {
            log.info("到达超时时间, 结束录制");
            record.set(false);
        }, timeout + 60, TimeUnit.SECONDS);
        long maxTime = TimeUnit.SECONDS.toMicros(timeout);
        try {
            Frame frame;
            while (!recorder.isCloseOutputStream() && record.get() && (frame = grabber.grab()) != null) {
                long micros = TimeUnit.MICROSECONDS.toMicros(recorder.getTimestamp());
                if(micros >= maxTime){
                    break;
                }
                recorder.record(frame);
            }
            grabber.close();
            recorder.close();
        } catch (FFmpegFrameRecorder.Exception | FrameGrabber.Exception e) {
            throw new RuntimeException(e);
        }

        log.info("结束录制");
        InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
        try {
            IoUtil.copy(inputStream, outputStream);
        } catch (Exception ignore) {
        }
        log.info("临时文件 {} 写入 响应 完成", file);
        log.info("删除临时文件 {} {}", file, file.delete());
        outputStream.close();

    }
}
